ISpy = LibStub("AceAddon-3.0"):NewAddon("ISpy", "AceConsole-3.0", "AceEvent-3.0")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ISpy", true)
local ISpy_MESSAGECOLOR = "ffffff00"

function ISpy:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ISpyDB", ISpy_DEFAULTS, "Default")
	
	self.options = ISpy_Options
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.inline = true
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ISpy", self.options, "ispy")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ISpy", "ISpy")
	
	self:RegisterChatCommand("ispy", "ChatCommand")
end

function ISpy:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self.options.args.otheroptions.args.enable.hidden = true
	print("|c"..ISpy_MESSAGECOLOR..L["ISpy"].."|r: "..L["Enabled"].." - "..L["Type \"/ispy\" for options or \"/ispy help\" for console commands"])
end

function ISpy:OnDisable()
	self.options.args.otheroptions.args.enable.hidden = false
	print("|c"..ISpy_MESSAGECOLOR..L["ISpy"].."|r: "..L["Disabled"])
end

function ISpy:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		if input:trim() == "help" then input = "" end
		LibStub("AceConfigCmd-3.0").HandleCommand(self, "ispy", "ISpy", input)
	end
end

function ISpy:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local outmsg = false;

	if self.db.profile.onlyingroup and GetNumRaidMembers() == 0 and GetNumPartyMembers() == 0 then return end
	
	local timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellID, spellName, _, extraskillID, extraskillname = ...;

	if eventType ~= "SPELL_INTERRUPT" and eventType ~= "SPELL_STOLEN" and eventType ~= "SPELL_DISPELL" then return end
	
	if eventType == "SPELL_INTERRUPT" and not self.db.profile.interrupt then return end
	if eventType == "SPELL_STOLEN" and not self.db.profile.steal then return end
	if eventType == "SPELL_DISPELL" then
		local enemy = UnitIsEnemy(sourceGUID, destGUID)
		if self.db.profile.debuff_friendly and enemy then return end
		if self.db.profile.debuff_hostile and not enemy then return end
	end
	if not self.db.profile.pvp then
		for i,v in pairs(self.db.profile.excludedzones) do
			if GetRealZoneText() == v then
				return
			end
		end
	end

	local grouped = UnitPlayerOrPetInParty(sourceName) or UnitPlayerOrPetInRaid(sourceName) or UnitPlayerOrPetInParty(destName) or UnitPlayerOrPetInRaid(destName) or UnitInParty(sourceName) or UnitInParty(destName)
	
	if self.db.profile.onlygroup and not grouped then
		return
	end
	
	if (eventType == "SPELL_INTERRUPT") then outmsg = L["Interrupt"] end
	if (eventType == "SPELL_STOLEN") then outmsg = L["Stolen"] end
	if (eventType == "SPELL_DISPEL") then outmsg = L["Dispell"] end

	if outmsg then
		local outmsgprv = "|c"..ISpy_MESSAGECOLOR..outmsg.."|r: "..sourceName.." " ..GetSpellLink(spellID).. " > "..destName.." "..GetSpellLink(extraskillID)
		outmsg = outmsg..": "..sourceName.." " ..GetSpellLink(spellID).. " > "..destName.." "..GetSpellLink(extraskillID)
		
		if self.db.profile.private then print(outmsgprv) end
		if self.db.profile.say then SendChatMessage(outmsg, "SAY") end
		if self.db.profile.yell then SendChatMessage(outmsg, "YELL") end
		if self.db.profile.guild then SendChatMessage(outmsg, "GUILD") end
		if self.db.profile.officer then SendChatMessage(outmsg, "OFFICER") end
		
		if self.db.profile.battleground and UnitInBattleground("player") then
			SendChatMessage(outmsg, "BATTLEGROUND")
		end
		
		if self.db.profile.whisper and self.db.profile.playername ~= "" then
			SendChatMessage(outmsg, "WHISPER", nil, self.db.profile.playername)
		end
		
		if self.db.profile.channel then
			local index = GetChannelName(self.db.profile.channelname)
			if index > 0 then
				SendChatMessage(outmsg, "CHANNEL", nil, index);
			end
		end
		
		if GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0 then
			if self.db.profile.raidwarning and (IsRaidOfficer() or IsRaidLeader()) and GetNumRaidMembers() > 0 then
				SendChatMessage(outmsg, "RAID_WARNING")
			elseif self.db.profile.raid and GetNumRaidMembers() > 0 then
				SendChatMessage(outmsg, "RAID")
			elseif self.db.profile.party and GetNumPartyMembers() > 0 then
				SendChatMessage(outmsg, "PARTY")
			end
		end
	end
end